#ifndef R2C2_3D_CATALOGUE_H_
#define R2C2_3D_CATALOGUE_H_

#include <msp/datafile/collection.h>
#include <msp/datafile/directorysource.h>
#include <msp/gl/mesh.h>
#include <msp/gl/resources.h>
#include "libr2c2/catalogue.h"
#include "typemap.h"

namespace R2C2 {

class ObjectType3D;

class Catalogue3D: public Msp::GL::Resources
{
private:
	typedef std::map<const ObjectType *, ObjectType3D *> ObjectMap;
	typedef std::map<const TrackAppearance *, Msp::GL::Mesh *> EndpointMap;

	Catalogue &catalogue;
	ObjectMap objects;
	EndpointMap endpoint_meshes;

public:
	Catalogue3D(Catalogue &);
	~Catalogue3D();

	const Catalogue &get_catalogue() const { return catalogue; }

	const ObjectType3D &get_3d(const ObjectType &);
	const ObjectType3D &get_3d(const ObjectType &) const;

	template<typename T>
	const typename TypeMap3D<T>::Type3D &get_3d(const T &ot) const
	{ return dynamic_cast<const typename TypeMap3D<T>::Type3D &>(get_3d(static_cast<const ObjectType &>(ot))); }

	template<typename T>
	const typename TypeMap3D<T>::Type3D &get_3d(const T &ot)
	{ return dynamic_cast<const typename TypeMap3D<T>::Type3D &>(get_3d(static_cast<const ObjectType &>(ot))); }

private:
	void build_endpoint_mesh(const TrackAppearance &);
public:
	const Msp::GL::Mesh &get_endpoint_mesh(const TrackAppearance &);
	const Msp::GL::Mesh &get_endpoint_mesh(const TrackAppearance &) const;
};

}

#endif
