#include <cmath>
#include <msp/gl/matrix.h>
#include <msp/gl/renderer.h>
#include "libr2c2/tracktype.h"
#include "endpoint.h"
#include "layout.h"
#include "path.h"
#include "track.h"
#include "tracktype.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Track3D::Track3D(Layout3D &l, Track &t):
	Object3D(l, t),
	GL::ObjectInstance(l.get_catalogue().get_3d(t.get_type()).get_object()),
	track(t),
	type(layout.get_catalogue().get_3d(track.get_type()))
{
	layout.get_scene().add(*this);

	const vector<TrackType::Endpoint> &type_eps = track.get_type().get_endpoints();
	const vector<Track *> &links = track.get_links();
	for(unsigned i=0; i<type_eps.size(); ++i)
	{
		if(!links[i] || links[i]>&track)
			endpoints.push_back(new Endpoint3D(*this, i));
		else
			endpoints.push_back(0);
	}

	track.signal_link_changed.connect(sigc::mem_fun(this, &Track3D::link_changed));
}

Track3D::~Track3D()
{
	layout.get_scene().remove(*this);

	for(vector<Endpoint3D *>::iterator i=endpoints.begin(); i!=endpoints.end(); ++i)
		delete *i;
}

Vector Track3D::get_node() const
{
	Geometry::BoundingBox<float, 3> bbox = track.get_type().get_shape()->get_axis_aligned_bounding_box();
	const Vector &minp = bbox.get_minimum_point();
	const Vector &maxp = bbox.get_maximum_point();

	return matrix*((minp+maxp)/2.0f+Vector(0, 0, 0.02));
}

void Track3D::setup_render(Msp::GL::Renderer &renderer, const GL::Tag &) const
{
	renderer.matrix_stack() *= matrix;
}

void Track3D::link_changed(unsigned i, Track *trk)
{
	if(!trk || trk>&track)
	{
		if(!endpoints[i])
			endpoints[i] = new Endpoint3D(*this, i);
	}
	else if(endpoints[i])
	{
		delete endpoints[i];
		endpoints[i] = 0;
	}
}

} // namespace R2C2
