#include <msp/strings/format.h>
#include "libr2c2/route.h"
#include "designer.h"
#include "routebar.h"

using namespace std;
using namespace Msp;
using namespace R2C2;

string route_name(Route *const &route)
{
	return route ? route->get_name() : "(new route)";
}

Routebar::Routebar(Designer &d):
	Toolbar("Route"),
	designer(d),
	routes(&route_name)
{
	pnl_content->add(*(drp_routes = new GLtk::Dropdown(routes)));
	drp_routes->set_tooltip("Select route to edit");
	drp_routes->signal_item_selected.connect(sigc::mem_fun(this, &Routebar::route_selected));

	GLtk::Button *btn;

	pnl_content->add(*(btn = new GLtk::Button("Del")));
	btn->set_style("red");
	btn->set_tooltip("Delete the current route");
	btn->signal_clicked.connect(sigc::mem_fun(this, &Routebar::delete_route_clicked));

	pnl_content->add(*(btn = new GLtk::Button("Name")));
	btn->set_tooltip("Rename the current route");
	btn->signal_clicked.connect(sigc::mem_fun(&designer, &Designer::rename_route));

	pnl_content->add(*(btn = new GLtk::Button("Add")));
	btn->set_tooltip("Add selected tracks to current route (A)");
	btn->signal_clicked.connect(sigc::mem_fun(&designer, &Designer::add_selection_to_route));

	designer.get_layout().signal_track_chain_added.connect(sigc::mem_fun(this, &Routebar::track_chain_added));
	designer.get_layout().signal_track_chain_removed.connect(sigc::mem_fun(this, &Routebar::track_chain_removed));

	const set<Route *> &lroutes = designer.get_layout().get_all<Route>();
	for(set<Route *>::const_iterator i=lroutes.begin(); i!=lroutes.end(); ++i)
	{
		routes.append(*i);
		(*i)->signal_name_changed.connect(sigc::bind(sigc::mem_fun(this, &Routebar::route_name_changed), *i));
	}

	routes.append(0);
}

void Routebar::route_selected(unsigned index)
{
	if(index==routes.size()-1)
	{
		Layout &rlayout = designer.get_layout();
		Route *route = new Route(rlayout);
		route->set_name(format("Route %d", index+1));
		designer.edit_route(route);

		for(unsigned i=0; i<routes.size(); ++i)
			if(routes.get(i)==route)
				drp_routes->set_selected_index(i);
	}
	else
		designer.edit_route(routes.get(index));
}

void Routebar::delete_route_clicked()
{
	Route *route = designer.get_current_route();
	designer.edit_route(0);
	delete route;
}

void Routebar::track_chain_added(TrackChain &tc)
{
	if(Route *r = dynamic_cast<Route *>(&tc))
	{
		r->signal_name_changed.connect(sigc::bind(sigc::mem_fun(this, &Routebar::route_name_changed), r));
		routes.insert(routes.size()-1, r);
	}
}

void Routebar::track_chain_removed(TrackChain &tc)
{
	if(Route *r = dynamic_cast<Route *>(&tc))
	{
		for(unsigned i=0; i<routes.size(); ++i)
			if(routes.get(i)==r)
			{
				routes.remove(i);
				break;
			}
	}
}

void Routebar::route_name_changed(const string &, Route *route)
{
	routes.refresh(route);
}
