#include <algorithm>
#include "libr2c2/block.h"
#include "selection.h"

using namespace std;
using namespace R2C2;
using namespace Msp;

Object *Selection::get_object() const
{
	if(objects.empty())
		return 0;
	else
		return *objects.begin();
}

void Selection::clear()
{
	objects.clear();
	signal_changed.emit();
}

void Selection::add_object(Object *o)
{
	if(objects.insert(o).second)
		signal_changed.emit();
}

void Selection::remove_object(Object *o)
{
	if(objects.erase(o))
		signal_changed.emit();
}

void Selection::toggle_object(Object *o)
{
	if(!objects.erase(o))
		objects.insert(o);

	signal_changed.emit();
}

void Selection::select_more()
{
	set<Object *> new_objects;
	for(set<Object *>::iterator i=objects.begin(); i!=objects.end(); ++i)
	{
		unsigned nls = (*i)->get_n_link_slots();
		for(unsigned j=0; j<nls; ++j)
			if(Object *obj = (*i)->get_link(j))
				new_objects.insert(obj);
	}

	bool changed = false;
	for(set<Object *>::iterator i=new_objects.begin(); i!=new_objects.end(); ++i)
		if(objects.insert(*i).second)
			changed = true;

	if(changed)
		signal_changed.emit();
}

void Selection::select_linked()
{
	bool changed = false;
	list<Object *> queue(objects.begin(), objects.end());

	while(!queue.empty())
	{
		Object *obj = queue.front();
		queue.pop_front();

		unsigned nls = obj->get_n_link_slots();
		for(unsigned j=0; j<nls; ++j)
			if(Object *linked = obj->get_link(j))
				if(objects.insert(linked).second)
				{
					queue.push_back(linked);
					changed = true;
				}
	}

	if(changed)
		signal_changed.emit();
}

void Selection::select_blocks()
{
	bool changed = false;
	for(set<Object *>::iterator i=objects.begin(); i!=objects.end(); ++i)
		if(Track *track = dynamic_cast<Track *>(*i))
		{
			const set<Track *> &btracks = track->get_block().get_tracks();
			for(set<Track *>::iterator j=btracks.begin(); j!=btracks.end(); ++j)
				if(objects.insert(*j).second)
					changed = true;
		}

	if(changed)
		signal_changed.emit();
}
