#include "client.h"
#include "train.h"

using namespace std;

namespace R2C2 {

NetTrain::NetTrain(Client &c, const TrainInfoPacket &pkt):
	client(c),
	loco_type(client.get_catalogue().get<VehicleType>(pkt.loco_type)),
	address(pkt.address),
	name(pkt.name),
	target_speed(0),
	reverse(false),
	functions(0)
{ }

void NetTrain::set_control(const string &c, float v)
{
	if(v==controls[c])
		return;

	TrainControlPacket pkt;
	pkt.address = address;
	pkt.control = c;
	pkt.value = v;
	client.send(pkt);
}

float NetTrain::get_control(const string &c) const
{
	map<string, float>::const_iterator i = controls.find(c);
	return i!=controls.end() ? i->second : 0;
}

void NetTrain::set_target_speed(float s)
{
	TrainAIControlPacket pkt;
	pkt.address = address;
	pkt.target_speed = s;
	pkt.reverse = reverse;
	client.send(pkt);
}

void NetTrain::set_reverse(bool r)
{
	TrainAIControlPacket pkt;
	pkt.address = address;
	pkt.target_speed = target_speed;
	pkt.reverse = r;
	client.send(pkt);
}

void NetTrain::set_function(unsigned i, bool set)
{
	TrainFunctionPacket pkt;
	pkt.address = address;
	pkt.functions = functions;
	if(set)
		pkt.functions |= 1<<i;
	else
		pkt.functions &= ~(1<<i);
	client.send(pkt);
}

void NetTrain::set_route(const string &r)
{
	if(r==route)
		return;

	TrainRoutePacket pkt;
	pkt.address = address;
	pkt.route = r;
	client.send(pkt);
}

void NetTrain::process_packet(const TrainControlPacket &pkt)
{
	controls[pkt.control] = pkt.value;
	signal_control_changed.emit(pkt.control, pkt.value);
}

void NetTrain::process_packet(const TrainFunctionPacket &pkt)
{
	unsigned old_funcs = functions;
	functions = pkt.functions;
	for(unsigned i=0; i<9; ++i)
		if(((functions^old_funcs)>>i)&1)
			signal_function_changed.emit(i, (functions>>i)&1);
}

void NetTrain::process_packet(const TrainAIControlPacket &pkt)
{
	if(pkt.target_speed!=target_speed)
	{
		target_speed = pkt.target_speed;
		signal_target_speed_changed.emit(target_speed);
	}

	if(pkt.reverse!=reverse)
	{
		reverse = pkt.reverse;
		signal_reverse_changed.emit(reverse);
	}
}

void NetTrain::process_packet(const TrainStatusPacket &pkt)
{
	status = pkt.status;
	signal_status_changed.emit(status);
}

void NetTrain::process_packet(const TrainRoutePacket &pkt)
{
	route = pkt.route;
	signal_route_changed.emit(route);
}

} // namespace R2C2
